
Const UIWINDOW_BACKGROUND	= 1
Const UIWINDOW_BODY		= 2
Const UIWINDOW_TITLEBAR		= 4
Const UIWINDOW_MINBTN		= 8
Const UIWINDOW_MAXBTN		= 16
Const UIWINDOW_CLOSEBTN		= 32
Const UIWINDOW_PINNED		= 64
Const UIWINDOW_CENTERED		= 128
Const UIWINDOW_DEFAULT		= 256

Type TUIWindow
	
	Global Gradient:TImage
	Global List:TList = CreateList()
	
	Field Text:String[2] 'Titletext, Statustext
	Field X:Int, Y:Int, W:Int, H:Int, A:Float, Flags:Int
	Field CX:Int, CY:Int, CW:Int, CH:Int
	Field Img:TImage, Panel:TUIPanel
	
	Function CreateGradient()
		Local PixMap:TPixmap = CreatePixmap( 1, 2, PF_I8 )
		WritePixel( PixMap, 0, 0, GenRGB( 255, 255, 255, 255 ) )
		WritePixel( PixMap, 0, 1, GenRGB( 195, 195, 195, 255 ) )
		
		TUIWindow.Gradient = LoadImage( PixMap )
	End Function
	
	Function Create:TUIWindow( StartX:Int, StartY:Int, Width:Int, Height:Int, TitleText:String, Flags:Int )
		
		Local A:TUIWindow = New TUIWindow
			A.X = StartX
			A.Y = StartY
			A.W = Width
			A.H = Height
			A.A = 1
			A.Text[0] = TitleText
			A.Flags = Flags
			
			If A.Flags&UIWINDOW_DEFAULT
				
				A.Flags = A.Flags + UIWINDOW_TITLEBAR | UIWINDOW_MINBTN | UIWINDOW_MAXBTN | UIWINDOW_CLOSEBTN | UIWINDOW_BODY
				
			EndIf
			
			If A.Flags&UIWINDOW_CENTERED
				A.X = GraphicsWidth() / 2 - A.W / 2
				A.Y = GraphicsHeight() / 2 - A.H / 2
			EndIf
			
			A.CalcClientArea()
			A.UpdateShape()
			
			A.Panel = TUIPanel.Create( A, 0, 0, A.CW, A.CH )
			
			ListAddLast( TUIWindow.List, A )
			
			Return A
	End Function
	
	Method BringToFront()
		
		TUIWindow.List.Remove( Self )
		TUIWindow.List.Addlast( Self )
		
	End Method
	
	Method CalcClientArea()
		
		CX = X
		CY = Y
		CW= W
		CH = H
		
		If Flags&UIWINDOW_BODY
			If Flags&UIWINDOW_TITLEBAR
				CX:+6
				CY:+26
				CW:-12
				CH:-32
			Else
				CX:+6
				CY:+6
				CW:-12
				CH:-12
			EndIf
		Else
			If Flags&UIWINDOW_TITLEBAR
				CY:+20
				CH:-20
			EndIf
		EndIf
		
		CX = CX - X
		CY = CY - Y
		
	End Method
	
	Method DrawBody()
		If Flags&UIWINDOW_TITLEBAR
			SetColor 147, 172, 206
			DrawImageRect( TUIWindow.Gradient, X, Y, W, H )
			SetColor 182, 199, 222
			DrawImageRect( TUIWindow.Gradient, X+5, Y+25, W-10, H-30 )
			SetColor 0, 0, 0
			DrawSquare( X, Y, W, H )
			DrawSquare( X+5, Y+25, W-10, H-30 )
			SetColor 255, 0, 255
			SetColor 255, 255, 255
		Else
			SetColor 147, 172, 206
			DrawImageRect( TUIWindow.Gradient, X, Y, W, H )
			SetColor 182, 199, 222
			DrawImageRect( TUIWindow.Gradient, X+5, Y+5, W-10, H-10 )
			SetColor 0, 0, 0
			DrawSquare( X, Y, W, H )
			DrawSquare( X+5, Y+5, W-10, H-10 )
			SetColor 255, 0, 255
			SetColor 255, 255, 255
		EndIf
	End Method
	
	Method DrawTitlebar()
		If Flags&UIWINDOW_BODY
			SetColor 147, 172, 206
				DrawImageRect( TUIWindow.Gradient, X, Y, W, 20 )
			SetColor 0, 0, 0
				Drawsquare( X, Y, W, 20 )
			SetColor 0, 0, 0
				DrawText( Text[0], X+5, Y+4 )
				DrawText( Text[0], X+3, Y+4 )
				DrawText( Text[0], X+4, Y+5 )
				DrawText( Text[0], X+4, Y+3 )
			SetColor 207, 218, 233
				DrawText( Text[0], X+4, Y+4 )
			SetColor 255, 255, 255
		Else
			SetColor 147, 172, 206
				DrawImageRect( TUIWindow.Gradient, X, Y, W, 20 )
			SetColor 0, 0, 0
				Drawsquare( X, Y, W, 20 )
			SetColor 0, 0, 0
				DrawText( Text[0], X+5, Y+4 )
				DrawText( Text[0], X+3, Y+4 )
				DrawText( Text[0], X+4, Y+5 )
				DrawText( Text[0], X+4, Y+3 )
			SetColor 207, 218, 233
				DrawText( Text[0], X+4, Y+4 )
			SetColor 255, 0, 255
				Plot X, Y + 20
				Plot X + W, Y + 20
			SetColor 255, 255, 255			
		EndIf
	End Method
	
	Method Drag()
		A = 0.5
		X:+ TMouse.XSpeed()
		Y:+ TMouse.YSpeed()
	End Method
	
	Method StopDrag()
		A = 1
		If X < 0
			X = 0
		EndIf
		If X+W > GraphicsWidth()
			X = GraphicsWidth() - W
		EndIf
		If Y < 0
			Y = 0
		EndIf
		If Y+H > GraphicsHeight()
			Y = GraphicsHeight()-H
		EndIf
	End Method
	
	Method UpdateShape()
		
		'Make screen 255, 0, 255 (Mask color)
		SetClsColor 255, 0, 255
		Cls
		
		'Move the new window to top left.
		SetOrigin( -X, -Y )
		
		'Flags
		If Flags&UIWINDOW_BODY
			DrawBody()
		EndIf
		
		If Flags&UIWINDOW_TITLEBAR
			DrawTitlebar()
		EndIf
				
		If Flags&UIWINDOW_MINBTN
		EndIf
		
		If Flags&UIWINDOW_MAXBTN
		EndIf
		
		If Flags&UIWINDOW_CLOSEBTN
		EndIf
		
		'Rounded corners.
		SetColor 255, 0, 255
			Plot X, Y
			Plot X,Y+H - 1
			Plot X+W - 1, Y
			Plot X+W - 1, Y+ H - 1
		SetColor 255, 255, 255
		
		'Save shape image.
		Img = LoadImage( MaskPixmap( GrabPixmap( 0, 0, W, H ), 255, 0, 255 ) )
		
		'give old positions back.
		SetOrigin( 0, 0 )
		
		'Give back old CLS color and CLS
		TGraphicsHandle.ResetCLSColor()
		Cls
		
	End Method
	
	Method Update()
		
		If A < 1 Then
			SetBlend( ALPHABLEND )
				SetAlpha( A )
		EndIf
		
		DrawImage( Img, X, Y )
		Panel.Update()
		
		If A < 1
				SetAlpha( 1 )
			SetBlend( MASKBLEND )
		EndIf
		
	End Method
	
	Function UpdateAll()
		Local A:TUIWindow
		For A = EachIn TUIWindow.List
			A.Update()
		Next
	End Function
	
	Method AddButton:TUIButton( X:Int, Y:Int, W:Int, H:Int, Text:String, Flags:Int )
		Return TUIButton.Create( Self, X, Y, W, H, Text, Flags )
	End Method
	
	Method AddProgBar:TUIProgBar( X:Int, Y:Int, W:Int, H:Int, Flags:Int )
		Return TUIProgBar.Create( Self, X, Y, W, H, Flags )
	End Method
	
	Method AddTextInput:TUITextInput( X:Int, Y:Int, W:Int, H:Int, Flags:Int )
		Return TUITextInput.Create( Self, X, Y, W, H, Flags )
	End Method
	
	Method ClientWidth:Int()
		Return CW
	End Method
	
	Method ClientHeight:Int()
		Return CH
	End Method
	
End Type